/* Wine Profiler
 *
 * Copyright 2000-2003 Codeweavers, Charles Loep
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef __WPROF_H__
#define __WPROF_H__

#include <unistd.h>
#include <sys/time.h>
#include <sys/resource.h>

#define XSTDCALL __attribute((__stdcall__))
#define XNOPROF  __attribute((no_instrument_function))

#define WPROF_MAGIC   0x4E4F4D57
#define WPROF_VERSION 0x02


typedef unsigned long long counter64_t;
typedef unsigned long      counter32_t;

typedef void * address32_t;


typedef struct arc_s {
    address32_t   addr;
    counter32_t   count;
    counter64_t   ticks;
    struct arc_s *children;
    struct arc_s *next;
} arc_t;


enum {
	WFLAG_NONE       = 0,
	WFLAG_CLOSED     = 1,
	WFLAG_HAS_RUSAGE = 2,
};

typedef struct wprof_header_s {
    unsigned int   magic;
    unsigned char  version;
    unsigned char  flags;
    unsigned char  resv[2];
    pid_t          pid;
    struct timeval time_start;
    struct timeval time_stop;
    struct rusage  rusage;
    int            exitcode;
    counter32_t    cpu_hz;
    unsigned int   vaddr;
    size_t         map_count;
    unsigned int   map_start;
    size_t         arc_count;
    arc_t          root[0];
} wprof_header_t;



typedef struct wprof_map_s {
    unsigned int addr_from;
    unsigned int addr_to;
    unsigned int offset;
    char         filename[256];
} wprof_map_t;



void wprof_exit_thread(int status);


#endif /* __WPROF_H__ */
