
#define __USE_GNU
#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#include <unistd.h>
#include <dlfcn.h>

#include "profiler.h"


#ifndef RTLD_NEXT
#define RTLD_NEXT ((void *)-1L)
#endif


static int wrappers_initialized = 0;
static void *(*real_dlopen)(const char *filename, int mode) = 0;
static int (*real_dlclose)(void *handle) = 0;
static void (*real_exit)(int status) __attribute__((__noreturn__)) = 0;
static void (*real__exit)(int status) = 0;

static void init_wrappers()
{
	real_dlopen = dlsym(RTLD_NEXT, "dlopen");
	real_dlclose = dlsym(RTLD_NEXT, "dlclose");
	real_exit = dlsym(RTLD_NEXT, "exit");
	real__exit = dlsym(RTLD_NEXT, "_exit");
}


void *dlopen(const char *filename, int flag)
{
	if(!wrappers_initialized)
		init_wrappers();

        printf("[%d] dlopen(%s, %d)\n", getpid(), filename, flag);
        return real_dlopen(filename, flag);
}

int dlclose(void *handle)
{
	if(!wrappers_initialized)
		init_wrappers();
	
	printf("[%d] dlclose(%p)\n", getpid(), handle);
	return real_dlclose(handle);
}

void exit(int status)
{
	if(!wrappers_initialized)
		init_wrappers();

	printf("[%d] exit(%d)\n", getpid(), status);
	wprof_exit_thread(status);

	real_exit(status);
}

void _exit(int status)
{
	if(!wrappers_initialized)
		init_wrappers();

	printf("[%d] _exit(%d)\n", getpid(), status);
	wprof_exit_thread(status);

	real__exit(status);
}


void pthread_exit(void *status )
{
	if(!wrappers_initialized)
		init_wrappers();

	printf("[%d] pthread_exit(%d)\n", getpid(), (int)status);
        wprof_exit_thread(0);

	//FIXME
	real__exit(0);
}
